class User < ActiveRecord::Base
  attr_accessible :email, :neptun, :encrypted_passwd, :role, :passwd, :passwd_confirmation
  attr_accessor :passwd
  has_many :subjects, :foreign_key => "lecturer_id"
  has_many :clazzs, :through => :subjects
  has_and_belongs_to_many :classes, :class_name => "Clazz"
  has_many :attended_subjects, :through => :classes, :source => :subject, :uniq => true
  
  validates_presence_of :neptun
  validates_uniqueness_of :neptun
  validates_length_of :neptun, :is => 6
  #validates_format_of :neptun, '/[a-zA-Z0-9]+/'
  validates_confirmation_of :passwd, :if => :password_required?
  
  def password_required?
    encrypted_passwd.blank? || !passwd.blank?
  end
  
  before_save :encrypt_passwd
  
  def self.encrypt(pass,salt)
    Digest::SHA1.hexdigest(salt+pass)
  end
  
  def encrypt_passwd
    return if passwd.blank?
    if new_record?
      self.salt = Digest::SHA1.hexdigest(Time.now.to_s+email+'--'+neptun)
    end
    self.encrypted_passwd = User.encrypt(passwd,salt)
  end
  
  def self.authenticate(username,password)
    user = User.find_by_neptun username
    user && user.authenticated?(password) ? user : nil
  end
  
  def authenticated?(password)
    encrypted_passwd == User.encrypt(password, salt)
  end
end
